//
//  chewMandelstam.c
//  etapi-xcode
//
//  Created by Vincent Mathieu on 4/29/18.
//
//



#include "chewM.h"
#include "modules.h"

// ********************************************************************
double complex numCM_c(double complex *par){
  // numerator of the Chew-Mandelstam
    
  double complex s  = par[0];
  int L = (int) creal(par[1]);
  double complex sR = par[2];
  double meta = creal(par[3]);
  
  double mpi2  = MPI*MPI;
  double meta2 = meta*meta;
  double complex slam = csqrt( lam(s,mpi2,meta2) );

  return cpow(slam, 2*L+1)/pow(s+sR,2*L+3);
}


// ********************************************************************
double intCM_re(double x, double complex *par){
  // real part integrand of the Chew-Mandelstam
  return creal( intCM_c(x,par) ) ;
}

// ********************************************************************
double intCM_im(double x, double complex *par){
  // real part integrand of the Chew-Mandelstam
  return cimag( intCM_c(x,par) ) ;
}


// ********************************************************************
double complex intCM_c(double x, double complex *par){
  // integrand of the Chew-Mandelstam
  
  double complex ur = 1.0 + I*0.0;
  double complex s  = par[0];
  //double complex sR = par[2];
  double meta = creal(par[3]);
  
  // change of variable bringing [s0, inf[ --> [0,1[
  double s0   = (MPI+meta)*(MPI+meta);
  double sp   = s0 + tan(M_PI/2.*x);
  double jaco = M_PI/2./cos(M_PI/2.*x)/cos(M_PI/2.*x);
  
  double complex parp[4] = {sp*ur, par[1], par[2], par[3]};
  
  return (numCM_c(parp) - numCM_c(par))*jaco/( sp*(sp-s) );
  
}

// ********************************************************************
double complex chewM_c(double complex *par){
  // compute the CM in the complex plane
  // the Chew-Mandelstam is written such that it is
  //    on sheet I    for Im(s)=>0
  //    on sheet II   for Im(s)<0
  
  
  double complex s  = par[0];
  double meta = creal(par[3]);
  
  
  double dok = 0.000000001; 					// precision
  double int_re = 0.0, int_im = 0.0;
  double s0 = (MPI+meta)*(MPI+meta);
  double complex ur = 1.0 + I*0.0;
  double complex ui = 0.0 + I*1.0;
  double complex cm;
  double complex integral = 0.0;
  
  
  int_re = int_adaptsimpson (*intCM_re, par, 0.0, 1.0, dok);
  int_im = int_adaptsimpson (*intCM_im, par, 0.0, 1.0, dok);
  integral = int_re * ur + int_im * ui;
  
  if (creal(s)>s0) {
    cm = ( s/M_PI*integral - numCM_c(par)/M_PI*log(s/s0-1.0) )*ur ;
    cm = cm  + ui*numCM_c(par);
  } else {
    cm = ( s/M_PI*integral - numCM_c(par)/M_PI*log(1.0-s/s0))*ur ;
  }
  
  
  return cm;
}



