//
//  main.c
//  PDwaves-PRL
//
//  Created by Vincent Mathieu on 11/18/18.
//  Copyright (c) 2018 Vincent Mathieu. All rights reserved.
//


#include "main.h"
#include "chewM.h"
#include "modules.h"
#include "partial-waves.h"



int main(int argc, const char * argv[]) {
  clock_t start = clock(), diff;
  printf("The code starts\n");
  
  // print the path
  char cwd[PATH_MAX];
  if (getcwd(cwd, sizeof(cwd)) != NULL) {
    //printf("Current working dir: %s\n", cwd);
  } else { perror("getcwd() error"); }
  
  
  
  // READ K-MATRIX PARAMETERS
  double param[12][4] = {{0.0}};
  char tmp[200], path[200] = "/Users/Vincent/Dropbox/mes articles/EtaPi/EtaPi-coupled/PDwaves-PRL/";
  FILE *fileParam = fopen(strcat(strcpy(tmp, path),"param-Kmatrix.txt"),"r");
  for (int i = 0; i<12; i++){
    fscanf(fileParam, "%lf %lf %lf %lf", &param[i][0], &param[i][1], &param[i][2], &param[i][3]);
  }
  fclose(fileParam);

  
  // File to print
  FILE *fileChew1 = fopen(strcat(strcpy(tmp, path),"chew1.txt"),"w");
  FILE *fileChew2 = fopen(strcat(strcpy(tmp, path),"chew2.txt"),"w");
  FILE *fileAmp1  = fopen(strcat(strcpy(tmp, path),"amp1.txt"),"w");
  FILE *fileAmp2  = fopen(strcat(strcpy(tmp, path),"amp2.txt"),"w");
  FILE *fileRes   = fopen(strcat(strcpy(tmp, path),"results.txt"),"w");

  // File to read the eta-pi mass values
  strcpy(tmp, "/Users/Vincent/Dropbox/mes articles/EtaPi/data/");
  FILE *fileAbs = fopen(strcat(tmp,"EtaPi-1mp.txt"),"r");
  
  double abs[56] = {0.0};
  double dummy[56][2] = {{0.0}};
  double t = -0.1, mom0, mom1;
  double complex par[4] = {0, 0, 1.0, 0};
  double complex ampP[2] = {0}, ampD[2] = {0};
  double complex chew[2] = {0.0 + I*0.0};
  
  
  for (int i=0; i < 56; i++) {
 
    // read the abscisse
    fscanf(fileAbs, "%lf %lf %lf", &abs[i], &dummy[i][0], &dummy[i][1]);
    par[0] = abs[i]*abs[i];
    
    // P-wave
    par[1] = 1;
    par[3] = META;  chew[0] = chewM_c(par);
    par[3] = METAP; chew[1] = chewM_c(par);
    Ampl(par[1], creal(par[0]), t, param, chew, ampP);
    
    // print Chew_Mandelstam in files
    fprintf(fileChew1, "%f  %f  %f  %f  %f\n", abs[i],
            creal(chew[0]), cimag(chew[0]), creal(chew[1]), cimag(chew[1]));
    
    
    // D-wave
    par[1] = 2;
    par[3] = META;   chew[0] = chewM_c(par);
    par[3] = METAP;  chew[1] = chewM_c(par);
    Ampl(par[1], creal(par[0]), t, param, chew, ampD);
    
    // print Chew_Mandelstam in files
    fprintf(fileChew2, "%f  %f  %f  %f  %f\n", abs[i],
            creal(chew[0]), cimag(chew[0]), creal(chew[1]), cimag(chew[1]));
    
    // print Amplitudes in files
    fprintf(fileAmp1, "%f  %f  %f  %f  %f\n", abs[i],
            creal(ampP[0]), cimag(ampP[0]), creal(ampP[1]), cimag(ampP[1]));
    fprintf(fileAmp2, "%f  %f  %f  %f  %f\n", abs[i],
            creal(ampD[0]), cimag(ampD[0]), creal(ampD[1]), cimag(ampD[1]));
    
    // break-up momenta
    mom0 = creal( csqrt( lam(abs[i]*abs[i], MPI*MPI, META*META)   )/(2*abs[i]) );
    mom1 = creal( csqrt( lam(abs[i]*abs[i], MPI*MPI, METAP*METAP) )/(2*abs[i]) );
    
    // print intensities and phase differences
    fprintf(fileRes, "%f  %f  %f  %f  %f %f %f\n", abs[i],
            mom0*pow(cabs(ampP[0]),2), mom1*pow(cabs(ampP[1]),2),
            mom0*pow(cabs(ampD[0]),2), mom1*pow(cabs(ampD[1]),2),
            carg(ampP[0])-carg(ampD[0]), carg(ampP[1])-carg(ampD[1]));
    
  }
  fclose(fileAbs);    fclose(fileRes);
  fclose(fileChew1);  fclose(fileChew2);
  fclose(fileAmp1);   fclose(fileAmp2);
  
  
  diff = clock() - start;
  long int msec = diff * 1000 / CLOCKS_PER_SEC;
  long int min = (msec/1000-(msec/1000)%60)/60;
  
  
  printf("\nTime taken: %ld minutes %ld seconds %ld milliseconds\n",min, (msec/1000)%60, msec%1000);
  return 0;
}












