//
//  modules.c
//  PDwaves-PRL
//
//  Created by Vincent Mathieu on 12/5/18.
//  Copyright (c) 2018 Vincent Mathieu. All rights reserved.
//

#include "modules.h"



/*
 * the function to integrate is f(x, *par)
 * x is the integration variable
 * par is a vector of parameters
 */


// ********************************************************************
double simpson(double (*f)(double, double complex*),double complex *par, double a,double b){
  return (b-a)/6*(f(a,par)+4*f((a+b)/2,par)+f(b,par));
}

// ********************************************************************
double int_adaptsimpson(double (*f)(double, double complex*),double complex *par, double a,double b,double tau){
  double m=(a+b)/2;
  double Sab=simpson(f,par,a,b),Sam=simpson(f,par,a,m),Smb=simpson(f,par,m,b);
  if (fabs(Sab-Sam-Smb)/15<tau)
    return Sam+Smb+(Sam+Smb-Sab)/15;
  else
    return int_adaptsimpson(f,par,a,m,tau)+int_adaptsimpson(f,par,m,b,tau);
}

// ********************************************************************
double complex lam(double complex a,double b, double c){
  return a*a + b*b + c*c - 2.0*(a*b + b*c + c*a);
}


//************************************************************************
void print_c(char* text, double complex x){
  // display a text followed by the complex number
  char tmp[200] = "";
  strcat(strcat(tmp, text), "(%f,%f)\n");
  printf(tmp, creal(x), cimag(x));
}


//************************************************************************
double ChebyPol(int n, double x){
  // return the Chebytchev polynomial of the first kind
  if(n<0){return 0;}
  if (fabs(x)<=1) {
    return cos(n*acos(x));
  } else {
    return  1/2*( pow(x-sqrt(x*x-1), n) + pow(x+sqrt(x*x-1), n) );
  }
}
