//
//  partial-waves.c
//  PDwaves-PRL
//
//  Created by Vincent Mathieu on 12/5/18.
//  Copyright (c) 2018 Vincent Mathieu. All rights reserved.
//

#include "partial-waves.h"
#include "modules.h"


//************************************************************************
void Ampl(int L, double s, double t, double par[12][4], double complex chew[2], double complex amp[2]){
  // return the amplitude of the partial wave L
  
  //double s01 =
  double complex mom0 = csqrt((s - pow(META  + MPI,2))*(s - pow(META  - MPI,2)))/sqrt(4*s);
  double complex mom1 = csqrt((s - pow(METAP + MPI,2))*(s - pow(METAP - MPI,2)))/sqrt(4*s);
  double complex momQ = csqrt(s*s + t*t + pow(MPI,4) - 2*(s*t + t*MPI*MPI + MPI*MPI*s)  )/sqrt(4*s);
  
  double invKmat[2][2] = {{0}};
  double complex deno[2][2] = {{0.0 + I*0.0}};
  double complex invdeno[2][2] = {{0.0 + I*0.0}};
  invKmatrix(L, s, par, invKmat); // compute the inverse K-matrix
  
  for (int i=0; i<2; i++) {
    for (int j=0; j<2; j++) {
      deno[i][j] = invKmat[i][j];
    }}
  deno[0][0] -= chew[0];
  deno[1][1] -= chew[1];
  
  double complex detD = deno[0][0]*deno[1][1] - deno[0][1]*deno[1][0]; // 2x2 determinant
  for (int i=0; i<2; i++) {
    for (int j=0; j<2; j++) {
      invdeno[i][j]  = pow(-1,i+j)*deno[1-i][1-j]/detD;
      //printf("inv = %f\n", creal(invdeno[i][j]));
    }}
  
  // the numerator
  double ai[4][4] = {{par[0][0], par[0][1], par[0][2], par[0][3]},
    {par[1][0], par[1][1], par[1][2], par[1][3]},
    {par[2][0], par[2][1], par[2][2], par[2][3]},
    {par[3][0], par[3][1], par[3][2], par[3][3]}};
  
  // shift the parameters between P and D waves;
  int ind = L*(L-1);
  
  double num[2] = {0};
  for (int i=0; i<2; i++) {
    for (int k=0; k<4; k++) {
      num[i] += ai[ind+i][k]*ChebyPol(k, s/(s+1));
    }}
  
  
  amp[0] = cpow(mom0,L)*cpow(momQ,L-1)*( num[0]*invdeno[0][0] + num[1]*invdeno[0][1]);
  amp[1] = cpow(mom1,L)*cpow(momQ,L-1)*( num[0]*invdeno[1][0] + num[1]*invdeno[1][1]);
  
}

//************************************************************************
void invKmatrix(int L, double s, double par[12][4], double invKmat[2][2]){
  /*  return a real matrix
   kmat[i][j] ; i,j = 0 --> eta-pi ; i,j = 1 --> eta'-pi
   the inverse of the kmatrix is actually return
   */
  double Kmat[2][2] = {{0}};
  double ci[4][3] = {{par[4][0],  par[4][1],  par[4][2]},
    {par[5][0],  par[5][1],  par[5][2]}};
  double di[4][3] = {{par[6][0],  par[6][1],  par[6][2]},
    {par[7][0],  par[7][1],  par[7][2]}};
  double gi[4][3] = {{par[8][0],  par[8][1],  par[8][2]},
    {par[9][0],  par[9][1],  par[9][2]},
    {par[10][0], par[10][1], par[10][2]},
    {par[11][0], par[11][1], par[11][2]}};
  
  // shift the parameters between P and D waves;
  int ind = L*(L-1);
  
  for (int i=0; i<2; i++) {
    for (int j=0; j<2; j++) {
      Kmat[i][j]  = gi[ind+0][i]*gi[ind+0][j]/(gi[ind+0][2] - s);
      Kmat[i][j] += gi[ind+1][i]*gi[ind+1][j]/(gi[ind+1][2] - s);
      Kmat[i][j] += ci[ind/2][i+j] + di[ind/2][i+j]*s;
    }}
  
  // 2x2 determinant
  double detK = Kmat[0][0]*Kmat[1][1] - Kmat[0][1]*Kmat[1][0];
  
  for (int i=0; i<2; i++) {
    for (int j=0; j<2; j++) {
      invKmat[i][j]  = pow(-1,i+j)*Kmat[1-i][1-j]/detK;
    }}
}
